@@ -25,7 +25,9 @@ from .CodeHelpers import (
     generateExpressionCode,
     withObjectCodeTemporaryAssignment,
 )
-from .templates.CodeTemplatesExceptions import template_publish_exception_to_handler
+from .templates.CodeTemplatesExceptions import (
+    template_publish_exception_to_handler,
+)
 
 
 def getExceptionIdentifier(exception_type):
@@ -149,13 +151,10 @@ def generateExceptionPublishCode(statement, emit, context):
         }
     )
 
+    # TODO: Make this one thing for performance with thread state shared, also for less code,
+    # then we should not make it in header anymore. Might be more scalable too.
     emit("NORMALIZE_EXCEPTION(&%s, &%s, &%s);" % (keeper_type, keeper_value, keeper_tb))
-
-    if python_version >= 300:
-        emit(
-            "PyException_SetTraceback(%s, (PyObject *)%s);" % (keeper_value, keeper_tb)
-        )
-
+    emit("ATTACH_TRACEBACK_TO_EXCEPTION_VALUE(%s, %s);" % (keeper_value, keeper_tb))
     emit("PUBLISH_EXCEPTION(&%s, &%s, &%s);" % (keeper_type, keeper_value, keeper_tb))
 
 
