@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,9 +21,12 @@ These are variable handling for classes and partially also Python2 exec
 statements.
 """
 
-from nuitka.nodes.shapes.BuiltinTypeShapes import ShapeTypeDict
+from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
 
-from .CodeHelpers import generateExpressionCode, withObjectCodeTemporaryAssignment
+from .CodeHelpers import (
+    generateExpressionCode,
+    withObjectCodeTemporaryAssignment,
+)
 from .Emission import SourceCodeCollector
 from .ErrorCodes import (
     getErrorExitBoolCode,
@@ -83,7 +86,7 @@ def generateLocalsDictSetCode(statement, emit, context):
     value_arg_name = context.allocateTempName("dictset_value", unique=True)
     generateExpressionCode(
         to_name=value_arg_name,
-        expression=statement.subnode_value,
+        expression=statement.subnode_source,
         emit=emit,
         context=context,
     )
@@ -94,7 +97,7 @@ def generateLocalsDictSetCode(statement, emit, context):
 
     locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
 
-    is_dict = locals_scope.getTypeShape() is ShapeTypeDict
+    is_dict = locals_scope.getTypeShape() is tshape_dict
 
     res_name = context.getIntResName()
 
@@ -133,7 +136,7 @@ def generateLocalsDictDelCode(statement, emit, context):
 
     dict_arg_name = locals_scope.getCodeName()
 
-    is_dict = locals_scope.getTypeShape() is ShapeTypeDict
+    is_dict = locals_scope.getTypeShape() is tshape_dict
 
     context.setCurrentSourceCodeReference(statement.getSourceReference())
 
@@ -194,7 +197,7 @@ def generateLocalsDictVariableRefOrFallbackCode(to_name, expression, emit, conte
         locals_scope = expression.getLocalsDictScope()
         locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
 
-        is_dict = locals_scope.getTypeShape() is ShapeTypeDict
+        is_dict = locals_scope.getTypeShape() is tshape_dict
 
         assert not context.needsCleanup(value_name)
 
@@ -226,7 +229,7 @@ def generateLocalsDictVariableRefCode(to_name, expression, emit, context):
 
     locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
 
-    is_dict = locals_scope.getTypeShape() is ShapeTypeDict
+    is_dict = locals_scope.getTypeShape() is tshape_dict
 
     if is_dict:
         template = template_read_locals_dict_without_fallback
@@ -266,12 +269,12 @@ def generateLocalsDictVariableCheckCode(to_name, expression, emit, context):
 
     locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
 
-    is_dict = locals_scope.getTypeShape() is ShapeTypeDict
+    is_dict = locals_scope.getTypeShape() is tshape_dict
 
     if is_dict:
         to_name.getCType().emitAssignmentCodeFromBoolCondition(
             to_name=to_name,
-            condition="PyDict_GetItem(%(locals_dict)s, %(var_name)s)"
+            condition="DICT_HAS_ITEM(%(locals_dict)s, %(var_name)s) == 1"
             % {
                 "locals_dict": locals_declaration,
                 "var_name": context.getConstantCode(constant=variable_name),
