@@ -22,7 +22,6 @@ much faster than general subscript lookups.
 """
 
 from nuitka import Options
-from nuitka.Constants import isIndexConstant
 
 from .CodeHelpers import (
     generateChildExpressionCode,
@@ -35,10 +34,8 @@ from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode
 
 def _decideIntegerSubscript(subscript):
     if subscript.isExpressionConstantRef():
-        constant = subscript.getConstant()
-
-        if isIndexConstant(constant):
-            constant_value = int(constant)
+        if subscript.isIndexConstant():
+            constant_value = subscript.getIndexValue()
 
             if abs(constant_value) < 2 ** 31:
                 return constant_value, True
@@ -71,7 +68,7 @@ def generateAssignmentSubscriptCode(statement, emit, context):
 
     old_source_ref = context.setCurrentSourceCodeReference(
         value.getSourceReference()
-        if Options.isFullCompat()
+        if Options.is_fullcompat
         else statement.getSourceReference()
     )
 
@@ -109,7 +106,7 @@ def generateDelSubscriptCode(statement, emit, context):
 
     old_source_ref = context.setCurrentSourceCodeReference(
         subscript.getSourceReference()
-        if Options.isFullCompat()
+        if Options.is_fullcompat
         else statement.getSourceReference()
     )
 
