@@ -46,8 +46,8 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
 
     def computeExpression(self, trace_collection):
         trace_collection.initIteratorValue(self)
-        value = self.getValue()
 
+        value = self.subnode_value
         return value.computeExpressionIter1(
             iter_node=self, trace_collection=trace_collection
         )
@@ -55,13 +55,13 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
     def computeExpressionIter1(self, iter_node, trace_collection):
         # Iteration over an iterator is that iterator.
 
-        return (self, "new_builtin", "Eliminated useless iterator creation.")
+        return self, "new_builtin", "Eliminated useless iterator creation."
 
     def getTypeShape(self):
-        return self.getValue().getTypeShape().getShapeIter()
+        return self.subnode_value.getTypeShape().getShapeIter()
 
     def computeExpressionNext1(self, next_node, trace_collection):
-        value = self.getValue()
+        value = self.subnode_value
 
         if value.isKnownToBeIterableAtMin(1) and value.canPredictIterationValues():
             result = wrapExpressionWithSideEffects(
@@ -70,8 +70,14 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
                 side_effects=value.getIterationValueRange(1, None),
             )
 
-            return result, "new_expression", "Pridicted 'next' value from iteration."
+            return False, (
+                result,
+                "new_expression",
+                "Predicted 'next' value from iteration.",
+            )
 
+        # TODO: This is only true for a few value types, use type shape to tell if
+        # it might escape or raise.
         self.onContentEscapes(trace_collection)
 
         # Any code could be run, note that.
@@ -80,42 +86,36 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
         # Any exception may be raised.
         trace_collection.onExceptionRaiseExit(BaseException)
 
-        return next_node, None, None
+        return True, (next_node, None, None)
 
     def isKnownToBeIterable(self, count):
         if count is None:
             return True
 
-        iter_length = self.getValue().getIterationLength()
+        iter_length = self.subnode_value.getIterationLength()
         return iter_length == count
 
     def isKnownToBeIterableAtMin(self, count):
         assert type(count) is int
 
-        iter_length = self.getValue().getIterationMinLength()
-        return iter_length is not None and count <= iter_length
-
-    def isKnownToBeIterableAtMax(self, count):
-        assert type(count) is int
-
-        iter_length = self.getValue().getIterationMaxLength()
+        iter_length = self.subnode_value.getIterationMinLength()
         return iter_length is not None and count <= iter_length
 
     def getIterationLength(self):
-        return self.getValue().getIterationLength()
+        return self.subnode_value.getIterationLength()
 
     def canPredictIterationValues(self):
-        return self.getValue().canPredictIterationValues()
+        return self.subnode_value.canPredictIterationValues()
 
     def getIterationValue(self, element_index):
-        return self.getValue().getIterationValue(element_index)
+        return self.subnode_value.getIterationValue(element_index)
 
     def getIterationHandle(self):
-        return self.getValue().getIterationHandle()
+        return self.subnode_value.getIterationHandle()
 
     def extractSideEffects(self):
         # Iterator making is the side effect itself.
-        value = self.getValue()
+        value = self.subnode_value
 
         if value.isCompileTimeConstant() and value.isKnownToBeIterable(None):
             return ()
@@ -123,13 +123,15 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
             return (self,)
 
     def mayHaveSideEffects(self):
-        if self.getValue().isCompileTimeConstant():
-            return not self.getValue().isKnownToBeIterable(None)
+        value = self.subnode_value
+
+        if value.isCompileTimeConstant():
+            return not value.isKnownToBeIterable(None)
 
         return True
 
     def mayRaiseException(self, exception_type):
-        value = self.getValue()
+        value = self.subnode_value
 
         if value.mayRaiseException(exception_type):
             return True
@@ -139,6 +141,11 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
 
         return True
 
+    def mayRaiseExceptionOperation(self):
+        value = self.subnode_value
+
+        return value.isKnownToBeIterable(None) is not True
+
     def onRelease(self, trace_collection):
         # print "onRelease", self
         pass
@@ -183,7 +190,9 @@ class StatementSpecialUnpackCheck(StatementChildHavingBase):
             trace_collection.onExceptionRaiseExit(BaseException)
 
         if iterator.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
+            from .NodeMakingHelpers import (
+                makeStatementExpressionOnlyReplacementNode,
+            )
 
             result = makeStatementExpressionOnlyReplacementNode(
                 expression=iterator, node=self
@@ -242,7 +251,8 @@ Determined iteration end check to always raise.""",
 
         return self, None, None
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "iteration check statement"
 
 
@@ -261,7 +271,8 @@ class ExpressionBuiltinIter2(ExpressionChildrenHavingBase):
             source_ref=source_ref,
         )
 
-    def getTypeShape(self):
+    @staticmethod
+    def getTypeShape():
         # TODO: This could be more specific.
         return tshape_iterator
 
@@ -279,7 +290,7 @@ class ExpressionAsyncIter(ExpressionBuiltinSingleArgBase):
     kind = "EXPRESSION_ASYNC_ITER"
 
     def computeExpression(self, trace_collection):
-        value = self.getValue()
+        value = self.subnode_value
 
         return value.computeExpressionAsyncIter(
             iter_node=self, trace_collection=trace_collection
@@ -293,23 +304,23 @@ class ExpressionAsyncIter(ExpressionBuiltinSingleArgBase):
         return None
 
     def getIterationLength(self):
-        return self.getValue().getIterationLength()
+        return self.subnode_value.getIterationLength()
 
     def extractSideEffects(self):
         # Iterator making is the side effect itself.
-        if self.getValue().isCompileTimeConstant():
+        if self.subnode_value.isCompileTimeConstant():
             return ()
         else:
             return (self,)
 
     def mayHaveSideEffects(self):
-        if self.getValue().isCompileTimeConstant():
-            return self.getValue().isKnownToBeIterable(None)
+        if self.subnode_value.isCompileTimeConstant():
+            return self.subnode_value.isKnownToBeIterable(None)
 
         return True
 
     def mayRaiseException(self, exception_type):
-        value = self.getValue()
+        value = self.subnode_value
 
         if value.mayRaiseException(exception_type):
             return True
@@ -322,16 +333,9 @@ class ExpressionAsyncIter(ExpressionBuiltinSingleArgBase):
     def isKnownToBeIterableAtMin(self, count):
         assert type(count) is int
 
-        iter_length = self.getValue().getIterationMinLength()
+        iter_length = self.subnode_value.getIterationMinLength()
         return iter_length is not None and iter_length < count
 
-    def isKnownToBeIterableAtMax(self, count):
-        assert type(count) is int
-
-        iter_length = self.getValue().getIterationMaxLength()
-
-        return iter_length is not None and count <= iter_length
-
     def onRelease(self, trace_collection):
         # print "onRelease", self
         pass
