@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,17 +20,22 @@
 """
 
 from nuitka import PythonOperators
+from nuitka.Errors import NuitkaAssumptionError
+from nuitka.PythonVersions import python_version
 
 from .ExpressionBases import ExpressionChildrenHavingBase
 from .NodeMakingHelpers import (
     makeConstantReplacementNode,
+    makeRaiseExceptionReplacementExpressionFromInstance,
     wrapExpressionWithSideEffects,
 )
-from .shapes.BuiltinTypeShapes import ShapeTypeBool
+from .shapes.BuiltinTypeShapes import tshape_bool, tshape_exception_class
 
 
 class ExpressionComparisonBase(ExpressionChildrenHavingBase):
     named_children = ("left", "right")
+    getLeft = ExpressionChildrenHavingBase.childGetter("left")
+    getRight = ExpressionChildrenHavingBase.childGetter("right")
 
     def __init__(self, left, right, source_ref):
         assert left.isExpression()
@@ -40,12 +45,13 @@ class ExpressionComparisonBase(ExpressionChildrenHavingBase):
             self, values={"left": left, "right": right}, source_ref=source_ref
         )
 
+    @staticmethod
+    def copyTraceStateFrom(source):
+        pass
+
     def getOperands(self):
         return (self.getLeft(), self.getRight())
 
-    getLeft = ExpressionChildrenHavingBase.childGetter("left")
-    getRight = ExpressionChildrenHavingBase.childGetter("right")
-
     def getComparator(self):
         return self.comparator
 
@@ -88,7 +94,7 @@ class ExpressionComparisonBase(ExpressionChildrenHavingBase):
         return self, None, None
 
     def computeExpressionOperationNot(self, not_node, trace_collection):
-        if self.getTypeShape() is ShapeTypeBool:
+        if self.getTypeShape() is tshape_bool:
             result = makeComparisonExpression(
                 left=self.subnode_left,
                 right=self.subnode_right,
@@ -96,11 +102,12 @@ class ExpressionComparisonBase(ExpressionChildrenHavingBase):
                 source_ref=self.source_ref,
             )
 
+            result.copyTraceStateFrom(self)
+
             return (
                 result,
                 "new_expression",
-                """\
-Replaced negated comparison '%s' with inverse comparison '%s'."""
+                """Replaced negated comparison '%s' with inverse comparison '%s'."""
                 % (self.comparator, result.comparator),
             )
 
@@ -122,6 +129,34 @@ class ExpressionComparisonRichBase(ExpressionComparisonBase):
     def getDetails(self):
         return {}
 
+    def copyTraceStateFrom(self, source):
+        self.type_shape = source.type_shape
+        self.escape_desc = source.escape_desc
+
+    def canCreateUnsupportedException(self):
+        return hasattr(self.subnode_left.getTypeShape(), "typical_value") and hasattr(
+            self.subnode_right.getTypeShape(), "typical_value"
+        )
+
+    def createUnsupportedException(self):
+        left = self.subnode_left.getTypeShape().typical_value
+        right = self.subnode_right.getTypeShape().typical_value
+
+        try:
+            self.getSimulator()(left, right)
+        except TypeError as e:
+            return e
+        else:
+            raise NuitkaAssumptionError(
+                "Unexpected no-exception doing comparison simulation",
+                self.operator,
+                self.simulator,
+                self.subnode_left.getTypeShape(),
+                self.subnode_right.getTypeShape(),
+                repr(left),
+                repr(right),
+            )
+
     def computeExpression(self, trace_collection):
         left = self.subnode_left
         right = self.subnode_right
@@ -140,6 +175,29 @@ class ExpressionComparisonRichBase(ExpressionComparisonBase):
         if exception_raise_exit is not None:
             trace_collection.onExceptionRaiseExit(exception_raise_exit)
 
+            if (
+                self.escape_desc.isUnsupported()
+                and self.canCreateUnsupportedException()
+            ):
+                result = wrapExpressionWithSideEffects(
+                    new_node=makeRaiseExceptionReplacementExpressionFromInstance(
+                        expression=self, exception=self.createUnsupportedException()
+                    ),
+                    old_node=self,
+                    side_effects=(self.subnode_left, self.subnode_right),
+                )
+
+                return (
+                    result,
+                    "new_raise",
+                    """Replaced comparator '%s' with %s %s arguments that cannot work."""
+                    % (
+                        self.comparator,
+                        self.subnode_left.getTypeShape(),
+                        self.subnode_right.getTypeShape(),
+                    ),
+                )
+
         if self.escape_desc.isValueEscaping():
             # The value of these nodes escaped and could change its contents.
             trace_collection.removeKnowledge(left)
@@ -160,6 +218,9 @@ class ExpressionComparisonRichBase(ExpressionComparisonBase):
             or self.subnode_right.mayRaiseException(exception_type)
         )
 
+    def mayRaiseExceptionBool(self, exception_type):
+        return self.type_shape.hasShapeSlotBool() is not True
+
     def mayRaiseExceptionComparison(self):
         return (
             self.escape_desc is None or self.escape_desc.getExceptionExit() is not None
@@ -253,7 +314,7 @@ class ExpressionComparisonNeq(ExpressionComparisonRichBase):
 
     @staticmethod
     def getComparisonShape(left_shape, right_shape):
-        return left_shape.getComparisonLteShape(right_shape)
+        return left_shape.getComparisonNeqShape(right_shape)
 
 
 class ExpressionComparisonIsIsNotBase(ExpressionComparisonBase):
@@ -270,8 +331,9 @@ class ExpressionComparisonIsIsNotBase(ExpressionComparisonBase):
     def getDetails(self):
         return {}
 
-    def getTypeShape(self):
-        return ShapeTypeBool
+    @staticmethod
+    def getTypeShape():
+        return tshape_bool
 
     def mayRaiseException(self, exception_type):
         return self.getLeft().mayRaiseException(
@@ -358,7 +420,7 @@ class ExpressionComparisonIs(ExpressionComparisonIsIsNotBase):
         )
 
 
-class ExpressionComparisonIsNOT(ExpressionComparisonIsIsNotBase):
+class ExpressionComparisonIsNot(ExpressionComparisonIsIsNotBase):
     kind = "EXPRESSION_COMPARISON_IS_NOT"
 
     comparator = "IsNot"
@@ -369,11 +431,7 @@ class ExpressionComparisonIsNOT(ExpressionComparisonIsIsNotBase):
         )
 
 
-class ExpressionComparisonExceptionMatch(ExpressionComparisonBase):
-    kind = "EXPRESSION_COMPARISON_EXCEPTION_MATCH"
-
-    comparator = "exception_match"
-
+class ExpressionComparisonExceptionMatchBase(ExpressionComparisonBase):
     def __init__(self, left, right, source_ref):
         ExpressionComparisonBase.__init__(
             self, left=left, right=right, source_ref=source_ref
@@ -382,11 +440,40 @@ class ExpressionComparisonExceptionMatch(ExpressionComparisonBase):
     def getDetails(self):
         return {}
 
+    @staticmethod
+    def getTypeShape():
+        return tshape_bool
+
     def getSimulator(self):
+        # TODO: Doesn't happen yet, but will once we trace exceptions.
         assert False
 
         return PythonOperators.all_comparison_functions[self.comparator]
 
+    def mayRaiseExceptionComparison(self):
+        if python_version < 300:
+            return False
+
+        # TODO: Add shape for exceptions.
+        type_shape = self.subnode_right.getTypeShape()
+
+        if type_shape is tshape_exception_class:
+            return False
+
+        return True
+
+
+class ExpressionComparisonExceptionMatch(ExpressionComparisonExceptionMatchBase):
+    kind = "EXPRESSION_COMPARISON_EXCEPTION_MATCH"
+
+    comparator = "exception_match"
+
+
+class ExpressionComparisonExceptionMismatch(ExpressionComparisonExceptionMatchBase):
+    kind = "EXPRESSION_COMPARISON_EXCEPTION_MISMATCH"
+
+    comparator = "exception_mismatch"
+
 
 class ExpressionComparisonInNotInBase(ExpressionComparisonBase):
     def __init__(self, left, right, source_ref):
@@ -399,8 +486,9 @@ class ExpressionComparisonInNotInBase(ExpressionComparisonBase):
     def getDetails(self):
         return {}
 
-    def getTypeShape(self):
-        return ShapeTypeBool
+    @staticmethod
+    def getTypeShape():
+        return tshape_bool
 
     def mayRaiseException(self, exception_type):
         left = self.getLeft()
@@ -415,7 +503,8 @@ class ExpressionComparisonInNotInBase(ExpressionComparisonBase):
 
         return right.mayRaiseExceptionIn(exception_type, left)
 
-    def mayRaiseExceptionBool(self, exception_type):
+    @staticmethod
+    def mayRaiseExceptionBool(exception_type):
         return False
 
     def computeExpression(self, trace_collection):
@@ -435,7 +524,7 @@ class ExpressionComparisonIn(ExpressionComparisonInNotInBase):
         )
 
 
-class ExpressionComparisonNOTIn(ExpressionComparisonInNotInBase):
+class ExpressionComparisonNotIn(ExpressionComparisonInNotInBase):
     kind = "EXPRESSION_COMPARISON_NOT_IN"
 
     comparator = "NotIn"
@@ -446,32 +535,23 @@ class ExpressionComparisonNOTIn(ExpressionComparisonInNotInBase):
         )
 
 
-def makeComparisonExpression(left, right, comparator, source_ref):
-    if comparator == "Is":
-        result = ExpressionComparisonIs(left=left, right=right, source_ref=source_ref)
-    elif comparator == "IsNot":
-        result = ExpressionComparisonIsNOT(
-            left=left, right=right, source_ref=source_ref
-        )
-    elif comparator == "In":
-        result = ExpressionComparisonIn(left=left, right=right, source_ref=source_ref)
-    elif comparator == "NotIn":
-        result = ExpressionComparisonNOTIn(
-            left=left, right=right, source_ref=source_ref
-        )
-    elif comparator == "Lt":
-        result = ExpressionComparisonLt(left=left, right=right, source_ref=source_ref)
-    elif comparator == "LtE":
-        result = ExpressionComparisonLte(left=left, right=right, source_ref=source_ref)
-    elif comparator == "Gt":
-        result = ExpressionComparisonGt(left=left, right=right, source_ref=source_ref)
-    elif comparator == "GtE":
-        result = ExpressionComparisonGte(left=left, right=right, source_ref=source_ref)
-    elif comparator == "Eq":
-        result = ExpressionComparisonEq(left=left, right=right, source_ref=source_ref)
-    elif comparator == "NotEq":
-        result = ExpressionComparisonNeq(left=left, right=right, source_ref=source_ref)
-    else:
-        assert False, comparator
+_comparator_to_nodeclass = {
+    "Is": ExpressionComparisonIs,
+    "IsNot": ExpressionComparisonIsNot,
+    "In": ExpressionComparisonIn,
+    "NotIn": ExpressionComparisonNotIn,
+    "Lt": ExpressionComparisonLt,
+    "LtE": ExpressionComparisonLte,
+    "Gt": ExpressionComparisonGt,
+    "GtE": ExpressionComparisonGte,
+    "Eq": ExpressionComparisonEq,
+    "NotEq": ExpressionComparisonNeq,
+    "exception_match": ExpressionComparisonExceptionMatch,
+    "exception_mismatch": ExpressionComparisonExceptionMismatch,
+}
 
-    return result
+
+def makeComparisonExpression(left, right, comparator, source_ref):
+    return _comparator_to_nodeclass[comparator](
+        left=left, right=right, source_ref=source_ref
+    )
