@@ -82,7 +82,7 @@ if python_version >= 300:
                     source_code=self.getSourceCode(),
                     globals_arg=self.getGlobals(),
                     locals_arg=self.getLocals(),
-                    source_ref=self.getSourceReference(),
+                    source_ref=self.source_ref,
                 )
 
                 del self.parent
@@ -124,7 +124,7 @@ if python_version < 300:
                     source_code=self.getSourceCode(),
                     globals_arg=self.getGlobals(),
                     locals_arg=self.getLocals(),
-                    source_ref=self.getSourceReference(),
+                    source_ref=self.source_ref,
                 )
 
                 del self.parent
@@ -240,15 +240,20 @@ class StatementLocalsDictSync(StatementChildHavingBase):
     named_child = "locals"
     getLocals = StatementChildHavingBase.childGetter("locals")
 
-    __slots__ = ("previous_traces", "variable_traces")
+    __slots__ = ("locals_scope", "previous_traces", "variable_traces")
 
     @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, locals_arg, source_ref):
+    def __init__(self, locals_scope, locals_arg, source_ref):
         StatementChildHavingBase.__init__(self, value=locals_arg, source_ref=source_ref)
 
         self.previous_traces = None
         self.variable_traces = None
 
+        self.locals_scope = locals_scope
+
+    def getDetails(self):
+        return {"locals_scope": self.locals_scope}
+
     def getPreviousVariablesTraces(self):
         return self.previous_traces
 
@@ -264,20 +269,17 @@ class StatementLocalsDictSync(StatementChildHavingBase):
         if provider.isCompiledPythonModule():
             return None, "new_statements", "Removed sync back to locals without locals."
 
-        self.previous_traces = trace_collection.onLocalsUsage(
-            self.getParentVariableProvider()
-        )
+        self.previous_traces = trace_collection.onLocalsUsage(self.locals_scope)
         if not self.previous_traces:
             return None, "new_statements", "Removed sync back to locals without locals."
 
         trace_collection.removeAllKnowledge()
-        self.variable_traces = trace_collection.onLocalsUsage(
-            self.getParentVariableProvider()
-        )
+        self.variable_traces = trace_collection.onLocalsUsage(self.locals_scope)
 
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
 
