@@ -59,17 +59,19 @@ class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
             self.getGeneratorRef().getFunctionBody().getClosureVariables()
         ):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
-            trace.addClosureUsage()
+            trace.addNameUsage()
 
             self.variable_closure_traces.append((closure_variable, trace))
 
         # TODO: Generator body may know something too.
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
-    def mayHaveSideEffects(self):
+    @staticmethod
+    def mayHaveSideEffects():
         return False
 
     def getClosureVariableVersions(self):
@@ -125,6 +127,10 @@ class StatementGeneratorReturn(StatementReturn):
     def __init__(self, expression, source_ref):
         StatementReturn.__init__(self, expression=expression, source_ref=source_ref)
 
+    @staticmethod
+    def isStatementGeneratorReturn():
+        return True
+
     def computeStatement(self, trace_collection):
         expression = trace_collection.onExpression(self.getExpression())
 
@@ -132,7 +138,9 @@ class StatementGeneratorReturn(StatementReturn):
             trace_collection.onExceptionRaiseExit(BaseException)
 
         if expression.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
+            from .NodeMakingHelpers import (
+                makeStatementExpressionOnlyReplacementNode,
+            )
 
             result = makeStatementExpressionOnlyReplacementNode(
                 expression=expression, node=self
@@ -160,10 +168,7 @@ Generator return value is always None.""",
         return self, None, None
 
     @staticmethod
-    def isStatementGeneratorReturn():
-        return True
-
-    def getStatementNiceName(self):
+    def getStatementNiceName():
         return "generator return statement"
 
 
@@ -179,5 +184,6 @@ class StatementGeneratorReturnNone(StatementReturnNone):
     def isStatementGeneratorReturn():
         return True
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "generator return statement"
