@@ -33,16 +33,16 @@ from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.ModuleRegistry import getModuleByName, getOwnerFromCodeName
 from nuitka.optimizations.TraceCollections import TraceCollectionModule
 from nuitka.PythonVersions import python_version
-from nuitka.SourceCodeReferences import SourceCodeReference, fromFilename
+from nuitka.SourceCodeReferences import fromFilename
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils.CStrings import encodePythonIdentifierToC
 from nuitka.utils.FileOperations import getFileContentByLine, relpath
 from nuitka.utils.ModuleNames import ModuleName
 
 from .Checkers import checkStatementsSequenceOrNone
-from .FutureSpecs import FutureSpec, fromFlags
+from .FutureSpecs import fromFlags
 from .IndicatorMixins import EntryPointMixin, MarkNeedsAnnotationsMixin
-from .LocalsScopes import getLocalsDictHandle, setLocalsDictType
+from .LocalsScopes import getLocalsDictHandle
 from .NodeBases import (
     ChildrenHavingMixin,
     ClosureGiverNodeMixin,
@@ -78,10 +78,6 @@ class PythonModuleBase(NodeBase):
     def isTopModule():
         return False
 
-    @staticmethod
-    def isInternalModule():
-        return False
-
     def attemptRecursion(self):
         # Make sure the package is recursed to if any
         package_name = self.module_name.getPackageName()
@@ -150,7 +146,7 @@ class PythonModuleBase(NodeBase):
         return None
 
     def getCompileTimeFilename(self):
-        """ The compile time filename for the module.
+        """The compile time filename for the module.
 
         Returns:
             Full path to module file at compile time.
@@ -160,10 +156,10 @@ class PythonModuleBase(NodeBase):
             all.
 
         """
-        return os.path.abspath(self.getSourceReference().getFilename())
+        return os.path.abspath(self.source_ref.getFilename())
 
     def getCompileTimeDirectory(self):
-        """ The compile time directory for the module.
+        """The compile time directory for the module.
 
         Returns:
             Full path to module directory at compile time.
@@ -215,9 +211,7 @@ class CompiledPythonModule(
     EntryPointMixin,
     PythonModuleBase,
 ):
-    """ Compiled Python Module
-
-    """
+    """Compiled Python Module"""
 
     # This one has a few indicators, pylint: disable=too-many-instance-attributes
 
@@ -252,7 +246,12 @@ class CompiledPythonModule(
 
         self.variables = {}
 
+        # Functions that have been used.
         self.active_functions = OrderedSet()
+
+        # Functions that should be visited again.
+        self.visited_functions = set()
+
         self.cross_used_functions = OrderedSet()
 
         self.used_modules = OrderedSet()
@@ -264,7 +263,14 @@ class CompiledPythonModule(
         self.source_code = None
 
         self.module_dict_name = "globals_%s" % (self.getCodeName(),)
-        setLocalsDictType(self.module_dict_name, "module_dict")
+
+        self.locals_scope = getLocalsDictHandle(
+            self.module_dict_name, "module_dict", self
+        )
+
+    @staticmethod
+    def isCompiledPythonModule():
+        return True
 
     def getDetails(self):
         return {
@@ -334,10 +340,8 @@ class CompiledPythonModule(
 
                 attrs = {"style": "filled"}
 
-                if variable_trace.hasDefiniteUsages():
+                if variable_trace.getUsageCount():
                     attrs["color"] = "blue"
-                elif variable_trace.hasPotentialUsages():
-                    attrs["color"] = "yellow"
                 else:
                     attrs["color"] = "red"
 
@@ -355,10 +359,6 @@ class CompiledPythonModule(
 
         return graph
 
-    @staticmethod
-    def isCompiledPythonModule():
-        return True
-
     def getSourceCode(self):
         if self.source_code is not None:
             return self.source_code
@@ -382,7 +382,7 @@ class CompiledPythonModule(
     def hasVariableName(self, variable_name):
         return variable_name in self.variables or variable_name in self.temp_variables
 
-    def getVariables(self):
+    def getProvidedVariables(self):
         return self.variables.values()
 
     def getFilename(self):
@@ -434,6 +434,9 @@ class CompiledPythonModule(
         self.used_modules = OrderedSet()
         self.active_functions = OrderedSet()
 
+    def restartTraversal(self):
+        self.visited_functions = set()
+
     def addUsedModule(self, key):
         self.used_modules.add(key)
 
@@ -441,7 +444,7 @@ class CompiledPythonModule(
         return self.used_modules
 
     def addUsedFunction(self, function_body):
-        assert function_body in self.getFunctions()
+        assert function_body in self.getFunctions(), function_body
 
         assert (
             function_body.isExpressionFunctionBody()
@@ -451,9 +454,10 @@ class CompiledPythonModule(
             or function_body.isExpressionAsyncgenObjectBody()
         )
 
-        result = function_body not in self.active_functions
-        if result:
-            self.active_functions.add(function_body)
+        self.active_functions.add(function_body)
+
+        result = function_body not in self.visited_functions
+        self.visited_functions.add(function_body)
 
         return result
 
@@ -492,6 +496,8 @@ class CompiledPythonModule(
         return result.replace(")", "").replace("(", "")
 
     def computeModule(self):
+        self.restartTraversal()
+
         old_collection = self.trace_collection
 
         self.trace_collection = TraceCollectionModule(self)
@@ -539,9 +545,7 @@ class CompiledPythonModule(
 
     @staticmethod
     def getFunctionVariablesWithAutoReleases():
-        """ Return the list of function variables that should be released at exit.
-
-        """
+        """Return the list of function variables that should be released at exit."""
         return ()
 
     def getOutlineLocalVariables(self):
@@ -555,7 +559,7 @@ class CompiledPythonModule(
         for outline in outlines:
             result.extend(outline.getUserLocalVariables())
 
-        return tuple(result)
+        return result
 
     def hasClosureVariable(self, variable):
         # Modules don't do this, pylint: disable=no-self-use,unused-argument
@@ -571,13 +575,8 @@ class CompiledPythonModule(
                 outline.removeUserVariable(variable)
                 break
 
-    @staticmethod
-    def getFunctionLocalsScope():
-        """ Modules have no locals scope. """
-        return None
-
-    def getModuleDictScope(self):
-        return getLocalsDictHandle(self.module_dict_name)
+    def getLocalsScope(self):
+        return self.locals_scope
 
 
 class CompiledPythonPackage(CompiledPythonModule):
@@ -632,9 +631,7 @@ def makeUncompiledPythonModule(
 
 
 class UncompiledPythonModule(PythonModuleBase):
-    """ Compiled Python Module
-
-    """
+    """Compiled Python Module"""
 
     kind = "UNCOMPILED_PYTHON_MODULE"
 
@@ -774,39 +771,6 @@ class PythonMainModule(CompiledPythonModule):
             return CompiledPythonModule.getOutputFilename(self)
 
 
-class PythonInternalModule(CompiledPythonModule):
-    """ The internal module is the parent for Python helpers.
-
-        For some operations, e.g. merging star arguments with
-        normal keyword arguments for a function call, there are
-        Python helpers.
-
-        This module is the home for these functions to live in,
-        but has no own module code.
-    """
-
-    kind = "PYTHON_INTERNAL_MODULE"
-
-    def __init__(self):
-        CompiledPythonModule.__init__(
-            self,
-            module_name=ModuleName("__internal__"),
-            is_top=False,
-            mode="compiled",
-            source_ref=SourceCodeReference.fromFilenameAndLine(
-                filename="internal", line=0
-            ),
-            future_spec=FutureSpec(),
-        )
-
-    @staticmethod
-    def isInternalModule():
-        return True
-
-    def getOutputFilename(self):
-        return "__internal"
-
-
 class PythonShlibModule(PythonModuleBase):
     kind = "PYTHON_SHLIB_MODULE"
 
@@ -834,7 +798,7 @@ class PythonShlibModule(PythonModuleBase):
         del self.used_modules
 
     def getFilename(self):
-        return self.getSourceReference().getFilename()
+        return self.source_ref.getFilename()
 
     def startTraversal(self):
         pass
@@ -851,7 +815,7 @@ class PythonShlibModule(PythonModuleBase):
     def _readPyPIFile(self):
         """ Read the .pyi file if present and scan for dependencies. """
 
-        # Complex stuff, pylint: disable=too-many-branches
+        # Complex stuff, pylint: disable=too-many-branches,too-many-statements
 
         if self.used_modules is None:
             pyi_filename = self.getPyIFilename()
@@ -876,10 +840,18 @@ class PythonShlibModule(PythonModuleBase):
                             assert parts[0] == "from"
                             assert parts[2] == "import"
 
-                            if parts[1] == "typing":
+                            origin_name = parts[1]
+
+                            if origin_name == "typing":
                                 continue
 
-                            pyi_deps.add(parts[1])
+                            if origin_name == ".":
+                                origin_name = self.getFullName()
+
+                            # TODO: Might want to add full relative import handling.
+
+                            if origin_name != self.getFullName():
+                                pyi_deps.add(origin_name)
 
                             imported = parts[3]
                             if imported.startswith("("):
@@ -887,7 +859,7 @@ class PythonShlibModule(PythonModuleBase):
                                 if not imported.endswith(")"):
                                     in_import = True
                                     imported = imported[1:]
-                                    in_import_part = parts[1]
+                                    in_import_part = origin_name
                                     assert in_import_part, (
                                         "Multiline part in file %s cannot be empty"
                                         % pyi_filename
@@ -903,7 +875,7 @@ class PythonShlibModule(PythonModuleBase):
                             for name in imported.split(","):
                                 if name:
                                     name = name.strip()
-                                    pyi_deps.add(parts[1] + "." + name)
+                                    pyi_deps.add(origin_name + "." + name)
 
                     else:  # In import
                         imported = line
@@ -926,6 +898,8 @@ class PythonShlibModule(PythonModuleBase):
     def getUsedModules(self):
         self._readPyPIFile()
 
+        assert "." not in self.used_modules, self
+
         return self.used_modules
 
     def getParentModule(self):
