@@ -28,16 +28,22 @@ from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import (
-    ExpressionMakeList,
-    ExpressionMakeTuple,
+    makeExpressionMakeList,
+    makeExpressionMakeTuple,
 )
+from nuitka.nodes.DictionaryNodes import StatementDictOperationSet
 from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.ImportNodes import (
     ExpressionImportModuleNameHard,
     ExpressionImportName,
 )
-from nuitka.nodes.ModuleAttributeNodes import ExpressionModuleAttributeFileRef
+from nuitka.nodes.ModuleAttributeNodes import (
+    ExpressionModuleAttributeFileRef,
+    ExpressionNuitkaLoaderCreation,
+)
 from nuitka.nodes.ModuleNodes import CompiledPythonPackage
+from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
+from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference
 
@@ -57,7 +63,7 @@ def _makeCall(module_name, import_name, attribute_name, source_ref, *args):
             attribute_name=attribute_name,
             source_ref=source_ref,
         ),
-        args=ExpressionMakeTuple(elements=args, source_ref=source_ref),
+        args=makeExpressionMakeTuple(elements=args, source_ref=source_ref),
         source_ref=source_ref,
     )
 
@@ -79,7 +85,7 @@ def createPathAssignment(package, source_ref):
                     attribute_name="dirname",
                     source_ref=source_ref,
                 ),
-                args=ExpressionMakeTuple(
+                args=makeExpressionMakeTuple(
                     elements=(
                         ExpressionModuleAttributeFileRef(
                             variable=package.getVariableForReference("__file__"),
@@ -123,7 +129,7 @@ def createPathAssignment(package, source_ref):
 
                 elements.append(path_part)
 
-        path_value = ExpressionMakeList(elements=elements, source_ref=source_ref)
+        path_value = makeExpressionMakeList(elements=elements, source_ref=source_ref)
 
     return StatementAssignmentVariableName(
         provider=package,
@@ -185,3 +191,20 @@ def createNamespacePackage(module_name, is_top, module_relpath):
     completeVariableClosures(package)
 
     return source_ref, package
+
+
+def createImporterCacheAssignment(package, source_ref):
+    return StatementDictOperationSet(
+        dict_arg=ExpressionImportModuleNameHard(
+            module_name="sys", import_name="path_importer_cache", source_ref=source_ref
+        ),
+        key=ExpressionSubscriptLookup(
+            expression=ExpressionVariableNameRef(
+                provider=package, variable_name="__path__", source_ref=source_ref
+            ),
+            subscript=makeConstantRefNode(constant=0, source_ref=source_ref),
+            source_ref=source_ref,
+        ),
+        value=ExpressionNuitkaLoaderCreation(provider=package, source_ref=source_ref),
+        source_ref=source_ref,
+    )
