@@ -43,7 +43,7 @@ from nuitka.nodes.NodeMakingHelpers import (
 from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableRef,
+    makeExpressionVariableRef,
 )
 from nuitka.PythonVersions import (
     getErrorMessageExecWithNestedFunction,
@@ -67,14 +67,14 @@ from .SyntaxErrors import raiseSyntaxError
 
 
 class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
-    """ Variable closure phase 1: Find assignments and early closure references.
+    """Variable closure phase 1: Find assignments and early closure references.
 
-        In class context, a reference to a variable must be obeyed immediately,
-        so that "variable = variable" takes first "variable" as a closure and
-        then adds a new local "variable" to override it from there on. For the
-        not early closure case of a function, this will not be done and only
-        assignments shall add local variables, and references will be ignored
-        until phase 2.
+    In class context, a reference to a variable must be obeyed immediately,
+    so that "variable = variable" takes first "variable" as a closure and
+    then adds a new local "variable" to override it from there on. For the
+    not early closure case of a function, this will not be done and only
+    assignments shall add local variables, and references will be ignored
+    until phase 2.
     """
 
     @staticmethod
@@ -90,7 +90,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
                 variable = node.takeVariableForClosure(variable_name=non_local_name)
 
-                node.registerProvidedVariable(variable)
+                node.getLocalsScope().registerClosureVariable(variable)
 
                 if variable.isModuleVariable() and user_provided:
                     raiseSyntaxError(
@@ -187,7 +187,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                                         source_ref=node.source_ref,
                                     ),
                                     StatementLocalsDictOperationSet(
-                                        locals_scope=provider.getFunctionLocalsScope(),
+                                        locals_scope=provider.getLocalsScope(),
                                         variable_name=variable_name,
                                         value=ExpressionTempVariableRef(
                                             variable=tmp_variable,
@@ -208,7 +208,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
                 else:
                     new_node = StatementLocalsDictOperationSet(
-                        locals_scope=provider.getFunctionLocalsScope(),
+                        locals_scope=provider.getLocalsScope(),
                         variable_name=variable_name,
                         value=node.subnode_source,
                         source_ref=node.source_ref,
@@ -241,7 +241,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                 # Classes always assign to locals dictionary except for closure
                 # variables taken.
                 new_node = StatementLocalsDictOperationDel(
-                    locals_scope=provider.getFunctionLocalsScope(),
+                    locals_scope=provider.getLocalsScope(),
                     variable_name=variable_name,
                     tolerant=node.tolerant,
                     source_ref=node.source_ref,
@@ -265,9 +265,8 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             parent.replaceChild(node, new_node)
 
     def onEnterNode(self, node):
-        # Mighty complex code with lots of branches and statements, but it
-        # couldn't be less without making it more difficult.
-        # pylint: disable=too-many-branches,too-many-statements
+        # Mighty complex code with lots of branches, but we aim to get rid of it.
+        # pylint: disable=too-many-branches
 
         if node.isExpressionVariableNameRef():
             provider = node.provider
@@ -279,10 +278,12 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     )
 
                     new_node = ExpressionLocalsVariableRefOrFallback(
-                        locals_scope=provider.getFunctionLocalsScope(),
+                        locals_scope=provider.getLocalsScope(),
                         variable_name=node.getVariableName(),
-                        fallback=ExpressionVariableRef(
-                            variable=variable, source_ref=node.source_ref
+                        fallback=makeExpressionVariableRef(
+                            variable=variable,
+                            locals_scope=provider.getLocalsScope(),
+                            source_ref=node.source_ref,
                         ),
                         source_ref=node.source_ref,
                     )
@@ -290,7 +291,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     variable.addVariableUser(provider)
                 else:
                     new_node = ExpressionLocalsVariableRef(
-                        locals_scope=provider.getFunctionLocalsScope(),
+                        locals_scope=provider.getLocalsScope(),
                         variable_name=node.getVariableName(),
                         source_ref=node.source_ref,
                     )
@@ -333,36 +334,6 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             # after they were declared, so we need to fix this up.
             if python_version >= 340:
                 self._handleQualnameSetup(node)
-        # Attribute access of names of class functions should be mangled, if
-        # they start with "__", but do not end in "__" as well.
-        elif (
-            node.isExpressionAttributeLookup()
-            or node.isStatementAssignmentAttribute()
-            or node.isStatementDelAttribute()
-        ):
-            attribute_name = node.getAttributeName()
-
-            if attribute_name.startswith("__") and not attribute_name.endswith("__"):
-                seen_function = False
-
-                current = node
-
-                while True:
-                    current = current.getParentVariableProvider()
-
-                    if current.isCompiledPythonModule():
-                        break
-
-                    if current.isExpressionClassBody():
-                        if seen_function:
-                            node.setAttributeName(
-                                "_%s%s"
-                                % (current.getName().lstrip("_"), attribute_name)
-                            )
-
-                        break
-
-                    seen_function = True
         # Check if continue and break are properly in loops. If not, raise a
         # syntax error.
         elif node.isStatementLoopBreak() or node.isStatementLoopContinue():
@@ -384,14 +355,14 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
 
 class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
-    """ Variable closure phase 2: Find assignments and references.
+    """Variable closure phase 2: Find assignments and references.
 
-        In class context, a reference to a variable must be obeyed immediately,
-        so that "variable = variable" takes first "variable" as a closure and
-        then adds a new local "variable" to override it from there on.
+    In class context, a reference to a variable must be obeyed immediately,
+    so that "variable = variable" takes first "variable" as a closure and
+    then adds a new local "variable" to override it from there on.
 
-        So, assignments for early closure, accesses will already have a
-        variable set now, the others, only in this phase.
+    So, assignments for early closure, accesses will already have a
+    variable set now, the others, only in this phase.
     """
 
     @staticmethod
@@ -435,19 +406,22 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
                 variable_name = node.getVariableName()
 
                 new_node = ExpressionLocalsVariableRefOrFallback(
-                    locals_scope=provider.getFunctionLocalsScope(),
+                    locals_scope=provider.getLocalsScope(),
                     variable_name=variable_name,
-                    fallback=ExpressionVariableRef(
+                    fallback=makeExpressionVariableRef(
                         variable=node.getParentModule().getVariableForReference(
                             variable_name
                         ),
+                        locals_scope=provider.getLocalsScope(),
                         source_ref=node.source_ref,
                     ),
                     source_ref=node.source_ref,
                 )
             else:
-                new_node = ExpressionVariableRef(
-                    variable=variable, source_ref=node.source_ref
+                new_node = makeExpressionVariableRef(
+                    variable=variable,
+                    locals_scope=provider.getLocalsScope(),
+                    source_ref=node.source_ref,
                 )
 
                 variable.addVariableUser(provider)
@@ -459,14 +433,14 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
 
 
 class VariableClosureLookupVisitorPhase3(VisitorNoopMixin):
-    """ Variable closure phase 3: Find errors and complete frame variables.
+    """Variable closure phase 3: Find errors and complete frame variables.
 
-        In this phase, we can do some fix-ups and find errors. We might e.g.
-        detect that a "del" was executed on a shared variable, which is not
-        allowed for Python 2.x, so it must be caught. The parsing wouldn't do
-        that.
+    In this phase, we can do some fix-ups and find errors. We might e.g.
+    detect that a "del" was executed on a shared variable, which is not
+    allowed for Python 2.x, so it must be caught. The parsing wouldn't do
+    that.
 
-        Also, frame objects for functions should learn their variable names.
+    Also, frame objects for functions should learn their variable names.
     """
 
     def onEnterNode(self, node):
@@ -492,10 +466,10 @@ can not delete variable '%s' referenced in nested scope"""
                     class_var = node.takeVariableForClosure("__class__")
                     class_var.addVariableUser(node)
 
-                    node.registerProvidedVariable(class_var)
+                    node.getLocalsScope().registerClosureVariable(class_var)
                     while node != class_var.getOwner():
                         node = node.getParentVariableProvider()
-                        node.registerProvidedVariable(class_var)
+                        node.getLocalsScope().registerClosureVariable(class_var)
 
 
 def completeVariableClosures(tree):
