@@ -34,10 +34,10 @@ from .Utils import getArchitecture, getOS, isWin32Windows
 
 
 def callExec(args):
-    """ Do exec in a portable way preserving exit code.
+    """Do exec in a portable way preserving exit code.
 
-        On Windows, unfortunately there is no real exec, so we have to spawn
-        a new process instead.
+    On Windows, unfortunately there is no real exec, so we have to spawn
+    a new process instead.
     """
 
     # On Windows os.execl does not work properly
@@ -90,9 +90,7 @@ def getExecutablePath(filename):
 
 
 def getPythonExePathWindows(search, arch):
-    """ Find Python on Windows.
-
-    """
+    """Find Python on Windows."""
 
     # Shortcuts for the default installation directories, to avoid going to
     # registry at all unless necessary. Any Python2 will do for Scons, so it
@@ -135,13 +133,13 @@ def getPythonExePathWindows(search, arch):
 
 
 def check_output(*popenargs, **kwargs):
-    """ Call a process and check result code.
+    """Call a process and check result code.
 
-        This is for Python 2.6 compatibility, which doesn't have that in its
-        standard library.
+    This is for Python 2.6 compatibility, which doesn't have that in its
+    standard library.
 
-        Note: We use same name as in Python stdlib, violating our rules to
-        make it more recognizable what this does.
+    Note: We use same name as in Python stdlib, violating our rules to
+    make it more recognizable what this does.
     """
 
     if "stdout" in kwargs:
@@ -162,13 +160,13 @@ def check_output(*popenargs, **kwargs):
 
 
 def check_call(*popenargs, **kwargs):
-    """ Call a process and check result code.
+    """Call a process and check result code.
 
-        Note: This catches the error, and makes it nicer, and an error
-        exit. So this is for tooling only.
+    Note: This catches the error, and makes it nicer, and an error
+    exit. So this is for tooling only.
 
-        Note: We use same name as in Python stdlib, violating our rules to
-        make it more recognizable what this does.
+    Note: We use same name as in Python stdlib, violating our rules to
+    make it more recognizable what this does.
     """
     try:
         subprocess.check_call(*popenargs, **kwargs)
@@ -177,9 +175,9 @@ def check_call(*popenargs, **kwargs):
 
 
 @contextmanager
-def withEnvironmentPathAdded(env_var_name, path):
-    if type(path) in (tuple, list):
-        path = os.pathsep.join(path)
+def withEnvironmentPathAdded(env_var_name, *paths):
+    paths = [path for path in paths if path]
+    path = os.pathsep.join(paths)
 
     if path:
         if str is not bytes and type(path) is bytes:
@@ -203,6 +201,8 @@ def withEnvironmentPathAdded(env_var_name, path):
 
 @contextmanager
 def withEnvironmentVarOverriden(env_var_name, value):
+    """ Change an environment and restore it after context. """
+
     if env_var_name in os.environ:
         old_value = os.environ[env_var_name]
     else:
@@ -222,8 +222,36 @@ def withEnvironmentVarOverriden(env_var_name, value):
         os.environ[env_var_name] = old_value
 
 
+@contextmanager
+def withEnvironmentVarsOverriden(mapping):
+    """ Change multiple environment variables and restore them after context. """
+
+    old_values = {}
+
+    for env_var_name, value in mapping.items():
+
+        if env_var_name in os.environ:
+            old_values[env_var_name] = os.environ[env_var_name]
+        else:
+            old_values[env_var_name] = None
+
+        if value is not None:
+            os.environ[env_var_name] = value
+        elif old_values[env_var_name] is not None:
+            del os.environ[env_var_name]
+
+    yield
+
+    for env_var_name, value in mapping.items():
+        if old_values[env_var_name] is None:
+            if value is not None:
+                del os.environ[env_var_name]
+        else:
+            os.environ[env_var_name] = old_values[env_var_name]
+
+
 def wrapCommandForDebuggerForExec(*args):
-    """ Wrap a command for system debugger to call exec
+    """Wrap a command for system debugger to call exec
 
     Args:
         args: (list of str) args for call to be debugged
@@ -250,7 +278,7 @@ def wrapCommandForDebuggerForExec(*args):
 
 
 def wrapCommandForDebuggerForSubprocess(*args):
-    """ Wrap a command for system debugger with subprocess module.
+    """Wrap a command for system debugger with subprocess module.
 
     Args:
         args: (list of str) args for call to be debugged
@@ -268,3 +296,69 @@ def wrapCommandForDebuggerForSubprocess(*args):
     args = args[0:1] + args[2:]
 
     return args
+
+
+def getPythonInstallPathWindows(supported, decider=lambda x: True):
+    """Find suitable Python on Windows.
+
+    Go over a list of provided, supported versions, and first try a few
+    guesses for their paths, then look into registry for user or system wide
+    installations.
+
+    The decider may reject a Python, then the search is continued, otherwise
+    it's returned.
+    """
+    # Many cases to deal with, due to arches, caching, etc.
+    # pylint: disable=too-many-branches
+
+    seen = set()
+
+    # Shortcuts for the default installation directories, to avoid going to
+    # registry at all unless necessary. Any Python2 will do for Scons, so it
+    # might be avoided entirely.
+    for search in supported:
+        candidate = r"C:\Python%s\python.exe" % search.replace(".", "")
+
+        if os.path.isfile(candidate):
+            install_dir = os.path.normcase(os.path.dirname(candidate))
+
+            if decider(install_dir):
+                return install_dir
+
+            seen.add(install_dir)
+
+    # Windows only code, pylint: disable=I0021,import-error,undefined-variable
+    if python_version < 300:
+        import _winreg as winreg  # pylint: disable=I0021,import-error,no-name-in-module
+    else:
+        import winreg  # pylint: disable=I0021,import-error,no-name-in-module
+
+    for search in supported:
+        for hkey_branch in (winreg.HKEY_LOCAL_MACHINE, winreg.HKEY_CURRENT_USER):
+            for arch_key in (0, winreg.KEY_WOW64_32KEY, winreg.KEY_WOW64_64KEY):
+                for suffix in "", "-32":
+                    try:
+                        key = winreg.OpenKey(
+                            hkey_branch,
+                            r"SOFTWARE\Python\PythonCore\%s%s\InstallPath"
+                            % (search, suffix),
+                            0,
+                            winreg.KEY_READ | arch_key,
+                        )
+
+                        install_dir = os.path.normpath(winreg.QueryValue(key, ""))
+                    except WindowsError:
+                        pass
+                    else:
+                        if install_dir not in seen:
+                            if decider(install_dir):
+                                return install_dir
+
+                            seen.add(install_dir)
+
+
+def getNullOutput():
+    try:
+        return subprocess.NULLDEV
+    except AttributeError:
+        return open(os.devnull, "wb")
