@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,7 +26,7 @@ allow to easily make checks on them.
 class ModuleName(str):
     @staticmethod
     def makeModuleNameInPackage(module_name, package_name):
-        """ Create a module name in a package.
+        """Create a module name in a package.
 
         Args:
             - module_name (str or ModuleName) module name to put below the package
@@ -51,31 +51,44 @@ class ModuleName(str):
         return "<ModuleName %s>" % str(self)
 
     def asString(self):
-        """ Get a simply str value.
+        """Get a simply str value.
 
-            Notes:
-                This should only be used to create constant values for code
-                generation, there is no other reason to lower the type of
-                these values otherwise.
+        Notes:
+            This should only be used to create constant values for code
+            generation, there is no other reason to lower the type of
+            these values otherwise.
         """
 
         return str(self)
 
     def getPackageName(self):
-        """ Get the package name if any.
+        """Get the package name if any.
 
-            Returns:
-                ModuleName of the containing package or None if already
-                top level.
+        Returns:
+            ModuleName of the containing package or None if already
+            top level.
         """
 
         return self.splitModuleBasename()[0]
 
-    def getBasename(self):
-        """ Get leaf name of the module without package part.
+    def getTopLevelPackageName(self):
+        """Get the top level package name.
 
-            Returns:
-                ModuleName without package.
+        Returns:
+            ModuleName of the top level name.
+        """
+        package_name = self.getPackageName()
+
+        if package_name is None:
+            return self
+        else:
+            return package_name.getTopLevelPackageName()
+
+    def getBasename(self):
+        """Get leaf name of the module without package part.
+
+        Returns:
+            ModuleName without package.
         """
         return self.splitModuleBasename()[1]
 
@@ -94,5 +107,12 @@ class ModuleName(str):
     def hasNamespace(self, package_name):
         return self == package_name or self.isBelowNamespace(package_name)
 
+    def hasOneOfNamespaces(self, *package_names):
+        for package_name in package_names:
+            if self.hasNamespace(package_name):
+                return True
+
+        return False
+
     def isBelowNamespace(self, package_name):
         return self.startswith(package_name + ".")
